import * as multer from 'multer';
import {
  BadRequestException,
  Controller,
  Get,
  Post,
  Response,
  Request,
  UploadedFile,
  UseGuards,
  UseInterceptors,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import { FileInterceptor } from '@nestjs/platform-express';
import { ApiBearerAuth, ApiBody, ApiConsumes, ApiTags } from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { Operador } from '../operador/entity/operador.entity';
import { UploadFileService } from './upload-file.service';
import { ValidarUsuarioService } from '../validar-usuario/validar-usuario.service';
import { MessageOutputDto } from '../dto/output/message.dto';

@Controller('upload-file')
@ApiTags('upload-file')
export class UploadFileController {
  constructor(
    private uploadFileService: UploadFileService,
    private validarUsuarioService: ValidarUsuarioService,
  ) {}

  @Serealize(MessageOutputDto)
  @Post('carga-masiva-equipos')
  @UseGuards(AuthGuard('jwt'))
  @UseInterceptors(FileInterceptor('csv'))
  @ApiBearerAuth('jwt')
  @ApiConsumes('multipart/form-data')
  @ApiBody({
    schema: {
      type: 'object',
      properties: { csv: { type: 'string', format: 'binary' } },
    },
  })
  cargaMasivaEquipos(
    @Request() req,
    @UploadedFile() file: Express.Multer.File,
  ) {
    const admin: Operador = req.user.operador;
    const path = file ? `${file.destination}/${file.filename}` : null;

    this.validarUsuarioService.validarAdmin(admin);
    if (!file) throw new BadRequestException('No se mandó ningún archivo.');
    return this.uploadFileService.createEquipos(path, admin);
  }

  @Serealize(MessageOutputDto)
  @Post('carga-masiva-usuarios')
  @UseGuards(AuthGuard('jwt'))
  @UseInterceptors(FileInterceptor('csv'))
  @ApiBearerAuth('jwt')
  @ApiConsumes('multipart/form-data')
  @ApiBody({
    schema: {
      type: 'object',
      properties: { csv: { type: 'string', format: 'binary' } },
    },
  })
  cargaMasivaUsuarios(
    @Request() req,
    @UploadedFile() file: Express.Multer.File,
  ) {
    const admin: Operador = req.user.operador;
    const path = file ? `${file.destination}/${file.filename}` : null;

    this.validarUsuarioService.validarAdmin(admin);
    if (!file) throw new BadRequestException('No se mandó ningún archivo.');
    return this.uploadFileService.createUsuarios(path, admin);
  }

  @Serealize(MessageOutputDto)
  @Post('upload-logo')
  @UseGuards(AuthGuard('jwt'))
  @UseInterceptors(
    FileInterceptor('logo', {
      storage: multer.diskStorage({ destination: './public' }),
    }),
  )
  @ApiBearerAuth('jwt')
  @ApiConsumes('multipart/form-data')
  @ApiBody({
    schema: {
      type: 'object',
      properties: { logo: { type: 'string', format: 'binary' } },
    },
  })
  uploadLogo(@Request() req, @UploadedFile() file: Express.Multer.File) {
    const admin: Operador = req.user.operador;

    this.validarUsuarioService.validarAdmin(admin);
    if (!file) throw new BadRequestException('No se mandó ningún archivo.');
    return this.uploadFileService.uploadLogo(file, admin);
  }
}
